@@ -1,5 +1,6 @@
 from django.contrib.contenttypes.models import ContentType
-from django.contrib.contenttypes import generic
+from django.contrib.contenttypes.fields import GenericForeignKey, GenericRelation
+
 from django.db import models
 
 
@@ -8,20 +9,31 @@ class Tag(models.Model):
     Tags have a descriptive slug, and are attached to an arbitrary object.
     """
     tag = models.SlugField()
-    content_type = models.ForeignKey(ContentType)
+    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
     object_id = models.PositiveIntegerField()
-    tagged_item = generic.GenericForeignKey('content_type', 'object_id')
+    tagged_item = GenericForeignKey('content_type', 'object_id')
 
     def __unicode__(self):
         return self.tag
 
 
+class Detachable(models.Model):
+    """
+    Model with an optional GenericForeignKey relation
+    """
+    name = models.CharField(max_length=50)
+    content_type = models.ForeignKey(
+        ContentType, null=True, blank=True, on_delete=models.CASCADE)
+    object_id = models.PositiveIntegerField(null=True, blank=True)
+    content_object = GenericForeignKey('content_type', 'object_id')
+
+
 class Bookmark(models.Model):
     """
     A URL bookmark that may have multiple tags attached.
     """
     url = models.URLField()
-    tags = generic.GenericRelation(Tag)
+    tags = GenericRelation(Tag)
 
     def __unicode__(self):
         return 'Bookmark: %s' % self.url
@@ -32,7 +44,12 @@ class Note(models.Model):
     A textual note that may have multiple tags attached.
     """
     text = models.TextField()
-    tags = generic.GenericRelation(Tag)
+    tags = GenericRelation(Tag)
 
     def __unicode__(self):
         return 'Note: %s' % self.text
+
+
+class NoteProxy(Note):
+    class Meta:
+        proxy = True
